/**
 * Copyright (C) 2006 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.inject.binder;

import com.google.inject.TypeLiteral;
import com.google.inject.Provider;
import com.google.inject.Key;

/**
 * Specifies an implementation for a binding.
 *
 * @author crazybob@google.com (Bob Lee)
 */
public interface BindingImplementationBuilder<T> {

  /**
   * Binds to instances of the given implementation class. The {@link
   * com.google.inject.Container} will inject {@code implementation} as
   * well. Sets the scope based on an annotation on the implementation
   * class if present.
   */
  BindingScopeBuilder to(Class<? extends T> implementation);

  /**
   * Binds to instances of the given implementation class. The {@link
   * com.google.inject.Container} will inject {@code implementation} as
   * well. Sets the scope based on an annotation on the implementation
   * class if present.
   */
  BindingScopeBuilder to(TypeLiteral<? extends T> implementation);

  /**
   * Binds to the given instance. Guice will inject the members of this
   * instance when it creates the container. See {@link
   * com.google.inject.Container#injectMembers(Object)}.
   */
  void toInstance(T instance);

  /**
   * Binds to instances generated by the given Provider. Guice will inject the
   * members of this provider instance when it creates the container. See
   * {@link com.google.inject.Container#injectMembers(Object)}.
   */
  BindingScopeBuilder toProvider(Provider<? extends T> provider);

  /**
   * Binds to instances from the provider bound to the given provider type.
   */
  BindingScopeBuilder toProvider(
      Class<? extends Provider<? extends T>> providerType);

  /**
   * Binds to instances from the provider bound to the given key.
   */
  BindingScopeBuilder toProvider(
      Key<? extends Provider<? extends T>> providerKey);
}
